#! /bin/sh
#
# $Id: bom.sh,v 1.32 2010/12/16 18:07:47 juli Exp $

BOM_ROOT=$HOME/buildomat
# XXX This needs to be per-world, per-kernel or per-host.
BOM_SRCDIR=$HOME/freebsd-src
BOM_MAKEFLAGS=-j4

# Nothing below this point should need adjusted.
BOM_CONFDIR=$BOM_ROOT/conf
BOM_HOSTCONFDIR=$BOM_CONFDIR/host
BOM_KERNELCONFDIR=$BOM_CONFDIR/kernel
BOM_WORLDCONFDIR=$BOM_CONFDIR/world

BOM_LIBDIR=$BOM_ROOT/lib

BOM_DEPLOYDIR=$BOM_ROOT/deploy
BOM_HOSTDEPLOYDIR=$BOM_DEPLOYDIR/host

BOM_OBJDIR=$BOM_ROOT/obj
BOM_KERNELOBJDIR=$BOM_OBJDIR/kernel
BOM_WORLDOBJDIR=$BOM_OBJDIR/world

BOM_HOSTLOGDIR=$BOM_HOSTDEPLOYDIR/log.d
BOM_KERNELLOGDIR=$BOM_KERNELOBJDIR/log.d
BOM_WORLDLOGDIR=$BOM_WORLDOBJDIR/log.d

BOM_JOBDIR=$BOM_ROOT/jobs

BOM_TMPDIR=`mktemp -d -t bom.$$.XXXXX` || exit $?

export PATH=/usr/local/bin:/usr/bin:/usr/sbin:/bin:/sbin

# Functions.
. $BOM_LIBDIR/util.sh
. $BOM_LIBDIR/check.sh
. $BOM_LIBDIR/enumerate.sh
. $BOM_LIBDIR/state.sh

# Verbs.
. $BOM_LIBDIR/build.sh
. $BOM_LIBDIR/deploy.sh
. $BOM_LIBDIR/list.sh
. $BOM_LIBDIR/status.sh

joblist() {
	for job_req in $BOM_JOBDIR/*.req; do
		if [ ! -e $job_req ]; then
			return
		fi
		job=`basename $job_req .req`
		if [ -e $BOM_JOBDIR/$job.running ]; then
			echo "running " $job_req
		else
			echo "request " $job_req
		fi
	done
}

jobrun() {
	debug Starting next job.
	run_job=""
	for job_req in $BOM_JOBDIR/*.req; do
		if [ ! -e $job_req ]; then
			continue
		fi
		job=`basename $job_req .req`
		if [ ! -e $BOM_JOBDIR/$job.running ]; then
			touch $BOM_JOBDIR/$job.running
			debug "Queueing job $job to run."
			run_job="$job"
			break
		fi
	done
	if [ ! -z "$run_job" ]; then
		job=$run_job
		job_req=$BOM_JOBDIR/$job.req

		debug "Starting job $job.  (Request file: $job_req)"
		sh <<_EOF_
		cd $BOM_ROOT
		sh bom.sh `cat $job_req` > $BOM_JOBDIR/$job.log 2>&1
		rm $BOM_JOBDIR/$job.*
_EOF_
		debug "Started job $job."
	fi
	debug Finished starting job.
}

jobsubmit() {
	job=`echo "$@" | md5 -q`
	if [ -e $BOM_JOBDIR/$job.req ]; then
		exit 0
	fi

	[ -d $BOM_JOBDIR ] || mkdir -p $BOM_JOBDIR
	echo "$@" > $BOM_JOBDIR/$job.req
}

mountobj() {
	sudo mdconfig -a -t swap -s 4096m -n | if read _unit; then
		if [ ! -d $BOM_OBJDIR ]; then
			mkdir $BOM_OBJDIR
		fi

		sudo newfs /dev/md$_unit
		sudo mount -o async /dev/md$_unit $BOM_OBJDIR

		sudo chown $USER $BOM_OBJDIR

		mount
	else
		error "Could not create memory-backed device for obj."
	fi
}

verb() {
	verb="$1"
	shift

	qualifier="$1"
	shift

	case "$qualifier" in
	-host | -h)
		${verb}_host "$1"
		;;
	-hosts | -all | -a)
		enumerate_apply hosts ${verb}_host
		;;
	-kernel | -k)
		${verb}_kernel "$1"
		;;
	-kernels)
		enumerate_apply kernels ${verb}_kernel
		;;
	-world | -w)
		${verb}_world "$1"
		;;
	-worlds)
		enumerate_apply worlds ${verb}_world
		;;
	*)
		[ -z "${qualifier}" ] || warning 'Unknown qualifier:' ${qualifier}
		info "usage: sh bom.sh ${verb} [-host host | -kernel kernel | -world world]"
		info "       sh bom.sh ${verb} [-hosts | -kernels | -worlds]"
		;;
	esac
}

verb_deploy() {
	qualifier="$1"
	shift

	case "$qualifier" in
	-host)
		deploy_host "$1"
		;;
	-hosts)
		deploy_hosts
		;;
	-kernel)
		deploy_kernel "$1"
		;;
	-kernels)
		deploy_kernels
		;;
	-world)
		deploy_world "$1"
		;;
	-worlds)
		deploy_worlds
		;;
	*)
		warning 'Unknown qualifier:' ${qualifier}
		info 'usage: sh bom.sh deploy [-host host | -kernel kernel | -world world]'
		info '       sh bom.sh deploy [-hosts | -kernels | -worlds]'
		;;
	esac
}

# Main.
main() {
	verb="$1"
	shift

	case "$verb" in
	build | b)
		verb build "$@"
		;;
	deploy | d)
		verb deploy "$@"
		;;
	list | ls | l)
		verb list "$@"
		;;
	status | s | st)
		verb status "$@"
		;;
	mountobj | mount)
		mountobj
		;;
	joblist)
		joblist
		;;
	jobrun)
		jobrun
		;;
	jobsubmit)
		jobsubmit "$@"
		;;
	*)
		[ -z "${verb}" ] || warning 'Unknown verb:' ${verb}
		info "usage: sh bom.sh [build | deploy | list | status] <qualifiers>"
		info "       sh bom.sh mountobj"
		info "       sh bom.sh [joblist | jobrun]"
		info "       sh bom.sh jobsubmit ..."
		;;
	esac

	exit 0
}

main "$@"
